const GObject = imports.gi.GObject;
const Gtk = imports.gi.Gtk;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Gdk = imports.gi.Gdk;
const ClutterGst = imports.gi.ClutterGst;

const Y901xWindow = imports.y901x_window.Y901xWindow;
const Y901xPreference = imports.y901x_preference.Y901xPreference;
const Y901xShortcutsWindow = imports.y901x_shortcut.Y901xShortcutsWindow;

const Y901X_VERSION = "1.2.beta7";

var Y901xApplication = GObject.registerClass({
    GTypeName: "Y901xApplication"
}, class Y901xApplication extends Gtk.Application {
    constructor(props={}) {
        super(props);
    }
    vfunc_startup() {
        //this.parent();
        super.vfunc_startup();
        // TaskBar Title
        GLib.set_prgname("Y901x");
        // Dark Theme
        let settings = Gtk.Settings.get_default();
        settings.gtk_application_prefer_dark_theme = true;
        // ApplicationMenu
        let menu = new Gio.Menu();
        menu.append("_Preference", "app.preference_action");
        menu.append("_Keyboard Shortcuts", "app.shortcut_action");
        menu.append("_About", "app.about_action");
        menu.append("_Quit", "app.quit_action");
        this.set_app_menu(menu);
        // Accel
        this.set_accels_for_action("app.shortcut_action", ["<Control>F1", "question"]);
        this.set_accels_for_action("app.quit_action", ["<Control>Q"]);
        // Action
        let preference_action  = new Gio.SimpleAction({name: "preference_action"});
        let shortcut_action    = new Gio.SimpleAction({name: "shortcut_action"});
        let about_action       = new Gio.SimpleAction({name: "about_action"});
        let quit_action        = new Gio.SimpleAction({name: "quit_action"});
        this.add_action(preference_action);
        this.add_action(shortcut_action);
        this.add_action(about_action);
        this.add_action(quit_action);
        preference_action.connect("activate", (action)=> {
            let dlg = new Y901xPreference(this.window);
            dlg.create();
            dlg.run();
            dlg.destroy();
        });
        shortcut_action.connect("activate", (action)=> {
            this.scwin.show_all();
        });
        about_action.connect("activate", ()=> {
            let d = new Gtk.AboutDialog({
                comments: "ClutterGst " + ClutterGst.VERSION_S + " Media Player",
                copyright: "Copyright(C) 2009-2017 sasakima-nao",
                //license: "GPL",
                license_type: Gtk.License.GPL_2_0,
                logo: this.window.get_icon(),
                program_name: "Y901x",
                version: Y901X_VERSION,
                website: "http://palepoli.skr.jp/"
            });
            d.set_transient_for(this.get_active_window());
            d.run();
            d.destroy();
        });
        quit_action.connect("activate", ()=> {
            this.quit();
        });
        //
        this.window = new Y901xWindow({application: this});
        this.window.create();
        this.scwin = new Y901xShortcutsWindow();
        this.scwin.create();
        this.window.set_help_overlay(this.scwin);
    }
    vfunc_open(files, hint) {
        let uri = files[0].get_uri();
        this.window.set_uri(uri);
        this.window.present();
    }
    vfunc_activate() {
        this.window.present();
    }
});
