const GObject = imports.gi.GObject;
const System = imports.system;
const Gtk = imports.gi.Gtk;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Gdk = imports.gi.Gdk;
const Clutter = imports.gi.Clutter;
const ClutterGst = imports.gi.ClutterGst;
const GtkClutter = imports.gi.GtkClutter;
const GdkPixbuf = imports.gi.GdkPixbuf;

const Y901xHeaderBar = imports.y901x_headerbar.Y901xHeaderBar;
const Y901xSeekBar = imports.y901x_seekbar.Y901xSeekBar;

const SEEKBAR_HEIGHT = 22;

var Y901xWindow = GObject.registerClass({
    GTypeName: "Y901xWindow"
}, class Y901xWindow extends Gtk.ApplicationWindow {
    constructor(props={}) {
        super(props);
    }
    create() {
        // Read Settings
        let settings = new Gio.Settings({schema: "org.sasakima.y901x"});
        let [w, h] = settings.get_value("window-size").deep_unpack();
        this.resize(w, h);
        this.types = settings.get_strv("content-type");
        this.set_size = settings.get_value("set-size").deep_unpack();
        // var
        this.position = 0;
        this.duration = 0;
        this.progress = 0;
        this.is_drag = false;
        this.is_fullscreen = false;
        this.is_playing = false;
        this.dirname = "";
        this.src_width = 0;
        this.src_height = 0;
        this.aspectrate ="source";
        this.rotate = "r0";
        // Timer
        this.main_timer = new Clutter.Timeline({
            duration: 200,
            loop: true
        });
        this.main_timer.connect("new-frame", ()=> {
            if (!this.is_drag) {
                this.position = this.player.get_position();
                this.duration = this.player.get_duration();
                this.progress = this.player.get_progress();
                this.seekbar.set_time(this.position, this.duration, this.progress);
            }
        });
        // Single Key
        let accelgroup = new Gtk.AccelGroup();
        accelgroup.connect(Gdk.KEY_space, 0, Gtk.AccelFlags.VISIBLE, ()=> {
            let b = this.player.get_playing() === false;
            this.player.set_playing(b);
        });
        accelgroup.connect(Gdk.KEY_Escape, 0, Gtk.AccelFlags.VISIBLE, ()=> {
            if (this.is_fullscreen)
                this.change_fullscreen();
            else
                this.close();
        });
        accelgroup.connect(Gdk.KEY_F11, 0, Gtk.AccelFlags.VISIBLE, ()=> {
            this.change_fullscreen();
        });
        accelgroup.connect(Gdk.KEY_F7, 0, Gtk.AccelFlags.VISIBLE, ()=> {
            this.headerbar.repeat_single.set_active(this.headerbar.repeat_single.get_active() == false);
        });
        accelgroup.connect(Gdk.KEY_F8, 0, Gtk.AccelFlags.VISIBLE, ()=> {
            this.headerbar.repeat_all.set_active(this.headerbar.repeat_all.get_active() == false);
        });
        this.add_accel_group(accelgroup);
        // Up Down Key
        this.set_events(Gdk.EventMask.KEY_PRESS_MASK | Gdk.EventMask.KEY_RELEASE_MASK | Gdk.EventMask.STRUCTURE_MASK );
        this.connect("key-press-event", (widget, event)=> {
            let [ok, keyval] = event.get_keyval();
            switch (keyval) {
            case Gdk.KEY_Up:
                let s = this.headerbar.prev_file();
                if (s) this.set_uri(GLib.build_filenamev([this.dirname, s]));
                break;
            case Gdk.KEY_Down:
                let d = this.headerbar.next_file();
                if (d) this.set_uri(GLib.build_filenamev([this.dirname, d]));
                break;
            case Gdk.KEY_1:
                this.change_box_size(1);
                break;
            case Gdk.KEY_2:
                this.change_box_size(2);
                break;
            case Gdk.KEY_3:
                this.change_box_size(3);
                break;
            case Gdk.KEY_4:
                this.change_box_size(0.5);
                break;
            case Gdk.KEY_5:
                this.change_box_size(1.5);
                break;
            case Gdk.KEY_6:
                this.change_box_size(2.5);
                break;
            case Gdk.KEY_7:
                this.change_box_size(7);
                break;
            case Gdk.KEY_8:
                this.change_box_size(8);
                break;
            case Gdk.KEY_9:
                this.change_box_size(9);
                break;
            }
        });
        // Actions
        let aspectRateActions = new Map([["source", "source"], ["hd", "16:9"], ["golden", "4:3"]]);
        for (let [menuName, menuLabel] of aspectRateActions) {
            let action = new Gio.SimpleAction({name: menuName});
            action.connect("activate", ()=> {
                if (this.aspectrate != action.name) {
                    this.aspectrate = action.name;
                    this.headerbar.aspectButton.label = menuLabel;
                    this.fit_video_size();
                }
            });
            this.add_action(action);
        }
        let rotateActions = new Map([["r0", 0], ["r90", 90], ["r180", 180], ["r270", 270]]);
        for (let [menuName, angle] of rotateActions) {
            let action = new Gio.SimpleAction({name: menuName});
            action.connect("activate", ()=> {
                if (this.rotate != action.name) {
                    this.rotate = action.name;
                    this.actor.set_rotation_angle(Clutter.RotateAxis.Z_AXIS, angle);
                    this.headerbar.rotateButton.label = `${angle}°`;
                    this.fit_video_size();
                }
            });
            this.add_action(action);
        }
        // Icon
        try {
            let iconpath = "/usr/share/y901x/data/icons/48x48/apps/y901x.png";
            this.set_icon(GdkPixbuf.Pixbuf.new_from_file(iconpath));
        }
        catch(e) {
            let noinstall = GLib.build_filenamev([GLib.path_get_dirname(System.programInvocationName), "data/icons/48x48/apps/y901x.png"]);
            this.set_icon(GdkPixbuf.Pixbuf.new_from_file(noinstall));
        }
        // HeaderBar
        this.headerbar = new Y901xHeaderBar({
            //has_subtitle: true,
            show_close_button: true
        });
        this.headerbar.create();
        this.set_titlebar(this.headerbar);
        this.headerbar.openButton.connect("clicked", ()=> {
            let dlg = new Gtk.FileChooserNative({
                title: "Open",
                transient_for: this, // parent
                action: Gtk.FileChooserAction.OPEN,
                accept_label: "_Open",
                cancel_label: "_Cancel"
            });
            let filter = new Gtk.FileFilter();
            filter.set_name("Support file");
            for (let i=0; i<this.types.length; i++)
                filter.add_mime_type(this.types[i]);
            dlg.add_filter(filter);
            if (this.dirname == "") {
                let sd = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_VIDEOS);
                if (sd != null)
                    dlg.set_current_folder(sd);
                else
                    dlg.set_current_folder(GLib.get_home_dir());
            } else {
                dlg.set_current_folder(this.dirname);
            }
            if (dlg.run() == Gtk.ResponseType.ACCEPT) {
                this.set_uri(dlg.get_filename());
            }
            dlg.destroy();
        });
        this.headerbar.treeview.connect("button-release-event", ()=> {
            let [res, model, it] = this.headerbar.selection.get_selected();
            if (res) {
            let s = model.get_value(it, 0);
                if (s != this.headerbar.get_title())
                    this.set_uri(GLib.build_filenamev([this.dirname, s]));
            }
        });
        // SeekBar
        this.seekbar = new Y901xSeekBar();
        this.seekbar.create();
        let gesture = new Clutter.GestureAction();
        gesture.connect("gesture-begin", (action, actor)=> {
            if (this.player.get_uri() != null) {
                this.is_drag = true;
                this.is_playing = this.player.get_playing();
                if (this.is_playing)
                    this.player.set_playing(false);
                let x = action.get_press_coords(0)[0] / this.seekbar.width;
                if (x < 0)
                    x = 0;
                else if (x > 1.0)
                    x = 1.0;
                this.player.set_progress(x);
                this.seekbar.set_progress(this.duration, x);
                return true;
            }
            return false;
        });
        gesture.connect("gesture-progress", (action, actor)=> {
            if (this.is_drag) {
                let x = action.get_motion_coords(0)[0] / this.seekbar.width;
                if (x < 0)
                    x = 0;
                else if (x > 1.0)
                    x = 1.0;
                this.player.set_progress(x);
                this.seekbar.set_progress(this.duration, x);
                return true;
            }
            return false;
        });
        gesture.connect("gesture-end", (action, actor)=> {
            if (this.is_drag) {
                this.is_drag = false;
                if (this.is_playing)
                    this.player.set_playing(true);
            }
        });
        this.seekbar.add_action(gesture);
        this.seekbar.set_reactive(true);
        // Player Create
        this.actor = new Clutter.Actor();
        this.actor.set_pivot_point(0.5, 0.5);
        this.actor.set_background_color(Clutter.Color.new(0, 0, 0, 255));
        this.content = new ClutterGst.Content();
        this.player = new ClutterGst.Playback();
        this.player.set_seek_flags(ClutterGst.SeekFlags.ACCURATE);
        // Repeat Select
        this.player.connect("eos", ()=> {
            this.main_timer.stop();
            if (this.headerbar.repeat_single.get_active()) {
                this.player.set_progress(0);
                this.player.set_playing(true);
            } else if (this.headerbar.repeat_all.get_active()) {
                let d = this.headerbar.next_file();
                if (d == null)
                    this.player.set_playing(false);
                else
                    this.set_uri(GLib.build_filenamev([this.dirname, d]));
            } else {
                this.player.set_playing(false);
            }
        });
        // Fullscreen Seekbar
        this.actor.connect("motion-event", (actor, event)=> {
            if (this.is_fullscreen) {
                let [x, y] = event.get_coords();
                if (this.get_allocated_height() - SEEKBAR_HEIGHT < y) {
                    if (!this.seekbar.visible) this.seekbar.show();
                } else {
                    if (this.seekbar.visible) this.seekbar.hide();
                }
            }
        });
        this.actor.set_reactive(true);
        // Player Set
        this.content.set_player(this.player);
        this.actor.set_content(this.content);
        let embed = new GtkClutter.Embed();
        this.stage = embed.get_stage();
        this.stage.add_child(this.actor);
        this.stage.add_child(this.seekbar);
        this.stage.set_background_color(Clutter.Color.new(0, 0, 0, 255));
        // box
        this.vbox = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL});
        this.vbox.pack_start(embed, true, true, 0);
        this.vbox.connect("size-allocate", (vbox, allocation)=> {
            let offset = SEEKBAR_HEIGHT;
            if (this.is_fullscreen) {
                offset = 0;
                this.seekbar.hide();
            } else {
                this.seekbar.show();
            }
            if (this.player.get_uri() != null) // beta4
                this.fit_video_size();
            this.seekbar.set_position(0, allocation.height - SEEKBAR_HEIGHT);
            this.seekbar.set_size(allocation.width, SEEKBAR_HEIGHT);
            this.seekbar.set_progress(this.duration, this.progress);
        });
        this.add(this.vbox);
        // Ready
        this.player.connect("ready", (player)=> {
            // Get Media About
            let vsink = player.get_video_sink();
            let it = vsink.iterate_pads(); // it == GstPad
            it.foreach((pad)=> {
                let caps = pad.get_current_caps();
                let struct = caps.get_structure(0);
                this.src_width = struct.get_int("width")[1];
                this.src_height = struct.get_int("height")[1];
            });
            this.fit_video_size();
            // Titlebar
            let uri = this.player.get_uri();
            let path = Gio.file_new_for_uri(uri).get_path();
            let title = GLib.path_get_basename(path);
            this.headerbar.set_title(title);
            // Timer
            this.main_timer.start();
            // Listup
            if (uri.substr(0, 7) == "file://") {
                let d = GLib.path_get_dirname(path);
                if (d != this.dirname) {
                    this.dirname = d;
                    this.headerbar.liststore.clear();
                    let files = [];
                    let x = Gio.file_new_for_path(d);
                    let enums = x.enumerate_children("standard::type,standard::name", Gio.FileQueryInfoFlags.NONE, null);
                    for (;;) {
                        let info = enums.next_file(null);
                        if (info == null)
                            break;
                        if (info.get_file_type() == Gio.FileType.REGULAR) {
                            let s = info.get_name();
                            if (s != null && s.indexOf(".") != 0 && s.substr(s.length - 1, 1) != "~") {
                                files.push(s);
                            }
                        }
                    }
                    files.sort(function(s1, s2) {
                        let cmpstr1 = GLib.utf8_collate_key_for_filename(s1, -1);
                        let cmpstr2 = GLib.utf8_collate_key_for_filename(s2, -1);
                        if (cmpstr1 < cmpstr2)
                            return -1;
                        else if (cmpstr1 > cmpstr2)
                            return 1;
                        return 0;
                    });
                    for (let i=0; i<files.length; i++) {
                        let name = GLib.build_filenamev([this.dirname, files[i]]);
                        let gvfs = Gio.File.new_for_path(name);
                        let info = gvfs.query_info(
                            Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                            Gio.FileQueryInfoFlags.NONE,
                            null
                        );
                        for (let j=0; j<this.types.length; j++) {
                            let mime = info.get_content_type();
                            if (mime == this.types[j]) {
                                let it = this.headerbar.liststore.append();
                                this.headerbar.liststore.set(it, [0], [files[i]]);
                            }
                        }
                    }
                }
                this.headerbar.select_text(title);
            }
        });
        // DnD
        this.drag_dest_set(
            Gtk.DestDefaults.MOTION | Gtk.DestDefaults.HIGHLIGHT | Gtk.DestDefaults.DROP,
            [Gtk.TargetEntry.new("text/uri-list", 0, 0)],
            Gdk.DragAction.MOVE
        );
        this.drag_dest_add_uri_targets();
        this.connect("drag-data-received", (widget, drag_context, x, y, data, info, time)=> {
            let drop = data.get_uris()[0];
            this.set_uri(drop);
            this.present();
        });
        // Double Click
        this.vbox.connect("button-press-event", (widget, event)=> {
            if (this.player.get_uri() == null) return false;
            if (event.get_event_type() == 5) {
                this.change_fullscreen();
            }
            if (event.get_button()[1] == 3) {
                let b = this.player.get_playing() === false;
                this.player.set_playing(b);
            }
            return true;
        });
        // Hide
        this.connect("hide", ()=> {
            this.player.set_playing(false);
            try {
                let settings = new Gio.Settings({schema:"org.sasakima.y901x"});
                let xy = new GLib.Variant("(ii)", this.get_size());
                settings.set_value("window-size", xy);
                //
                let builder = GLib.VariantBuilder.new(new GLib.VariantType("a(ii)"));
                for (let i=0; i<3; i++) {
                    let t = new GLib.Variant("(ii)", [ this.set_size[i][0], this.set_size[i][1] ]);
                    builder.add_value(t);
                }
                settings.set_value("set-size", builder.end());
                //
                settings.set_strv("content-type", this.types);
            }
            catch (e) {
            }
        });
        this.show_all();
    }
    // functions
    set_uri(str) {
        let uri = str;
        if (str.indexOf("//") == -1) {
            let f = Gio.File.new_for_path(str);
            uri = f.get_uri();
        }
        this.player.set_uri(uri);
        this.player.set_playing(true);
    }
    change_fullscreen() {
        this.is_fullscreen = this.is_fullscreen === false;
        if (this.is_fullscreen)
            this.fullscreen();
        else
            this.unfullscreen();
    }
    change_box_size(n) {
        if (this.player.get_uri() != null) {
            if (this.is_fullscreen)
                this.change_fullscreen();
            let w = Math.round(this.src_width * n);
            let h = Math.round(this.src_height * n) + SEEKBAR_HEIGHT;
            if (this.rotate == "r90" || this.rotate == "r270") {
                w = Math.round(this.src_height * n);
                h = Math.round(this.src_width * n) + SEEKBAR_HEIGHT;
            }
            if (n == 7) {
                w = this.set_size[0][0];
                h = this.set_size[0][1] + SEEKBAR_HEIGHT;
            } else if (n == 8) {
                w = this.set_size[1][0];
                h = this.set_size[1][1] + SEEKBAR_HEIGHT;
            } else if (n == 9) {
                w = this.set_size[2][0];
                h = this.set_size[2][1] + SEEKBAR_HEIGHT;
            }
            this.resize(w, h);
        }
    }
    fit_video_size() {
        if (this.src_width == 0 || this.src_height == 0)
            return;
        let allocation = this.vbox.get_allocation();
        let offset = SEEKBAR_HEIGHT;
        if (this.is_fullscreen)
            offset = 0;
        if (this.player.get_uri() != null) {
            let w = this.src_width;
            if (this.aspectrate == "hd")
                w = this.src_height *16 / 9;
            else if (this.aspectrate == "golden")
                w = this.src_height *4 / 3;
            let h = this.src_height;
            let aw = allocation.width;
            let ah = allocation.height - offset;
            let width, height, x, y = 0;
            // rotate
            if (this.rotate == "r90" || this.rotate == "r270") {
                if (ah * h > aw * w) {
                    width = w * aw / h;
                    height = ah;
                    x = (aw - width) / 2;
                    y = (ah - height) / 2;
                } else {
                    width = ah;
                    height = h * ah / w;
                    x = (aw - width) / 2;
                    y = (ah - height) / 2;
                }
            } else {
                if (aw * h > ah * w) {
                    width = w * ah / h;
                    height = ah;
                    x = (aw - width) / 2;
                    y = 0;
                } else {
                    width = aw;
                    height = h * aw / w;
                    x = 0;
                    y = (ah - height) / 2;
                }
            }
            this.actor.set_width(width);
            this.actor.set_height(height);
            this.actor.set_position(x, y);
        } else {
            this.actor.set_width(allocation.width);
            this.actor.set_height(allocation.height - offset);
        }
    }
});
